import 'dart:math';
import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_maps/maps.dart';

import 'package:latlong2/latlong.dart';

import '../data_classes.dart';

/// A class containing thickness data for each subdivision of the map.
class IceThicknessModel {
  IceThicknessModel(this.sub_div_id, this.thickness, this.color, this.savedColor);

  final String sub_div_id;
  final int thickness;
  Color color;
  final Color savedColor;
}

/// ChoroplethMap is a stateful widget that contains a choropleth map.
/// The map is created using the Syncfusion Flutter Maps library and
/// coordinates fetched from the server.
class ChoroplethMap extends StatefulWidget {
  const ChoroplethMap({Key? key,
    required this.relation,
    required this.measurements,
    required this.onSelectionChanged,
  }) : super(key: key);

  final Uint8List relation;
  final List<Measurement> measurements;
  final void Function(int selectedIndex) onSelectionChanged; // Callback function

  @override
  _ChoroplethMapState createState() => _ChoroplethMapState();
}

class _ChoroplethMapState extends State<ChoroplethMap> {
  int selectedIndex = -1;
  late MapShapeSource mapShapeSource;
  late final MapZoomPanBehavior _zoomPanBehavior = MapZoomPanBehavior();
  List<SubDiv> subdivisions = <SubDiv>[];
  int count = 0;


  @override
  void initState() {
    super.initState();
    // Create list of all subdivisions
    for (Measurement measurement in widget.measurements) {
      for (SubDiv subdivision in measurement.subDivs) {
        subdivisions.add(subdivision);
        print("SubDivID: ${subdivision.sub_div_id}");
        count++;
      }
    };

    // NB temporary filler
    for (var i = count; i < 250; i++) {
      SubDiv subdivision = SubDiv(
        sub_div_id: i.toString(),
        groupID: 0,
        minThickness: 0.0,
        avgThickness: 0.0,
        center: LatLng(0.0, 0.0),
        accuracy: 0.0,
        color: Colors.grey,
        savedColor: Colors.grey,
      );
      subdivisions.add(subdivision);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        SfMaps(
          layers: [
            MapShapeLayer(
              source: MapShapeSource.memory( // Map polygon
                widget.relation, // JSON coordinates from server
                shapeDataField: 'sub_div_id',
                dataCount: 250,
                primaryValueMapper: (int index) => subdivisions[index].sub_div_id,
                shapeColorValueMapper: (int index) => subdivisions[index].color,
              ),
              zoomPanBehavior: _zoomPanBehavior,
              strokeColor: Colors.blue.shade50,
              // Shape selection
              selectedIndex: selectedIndex,
              onSelectionChanged: (int index) {
                setState(() {
                  selectedIndex = index;
                  for (int i = 0; i < subdivisions.length; i++) {
                    subdivisions[i].color = i == index ? Colors.red : subdivisions[i].savedColor;
                  }
                });
                widget.onSelectionChanged(selectedIndex);
              },
            ),
          ],
        ),
      ],
    );
  }
}

